from __future__ import annotations
from threading import Lock, Thread
from typing import Optional
# -*- coding: utf-8 -*-
"""
Created on Mon Feb  7 19:50:47 2022

@author: krein_000
"""
from intern.gng_klassen import Bild, Text, Ereignisbehandlung
import time


class SingletonMeta(type):
    """ 
    Hilfsklasse zur Realisierung des Singleton-Musters, damit zu jeder Zeit nur ein einziges Objekt der Klasse Zeichenfenster existiert. 
    """
    _instanz: Optional[Singleton] = None
    _lock: Lock = Lock()


    def __call__(cls, *args, **kwargs):
        """ 
        Methode sorgt dafür, dass nur ein Objekt der Klasse ausgegeben wird.
        """
        with cls._lock:
            if not cls._instanz:
                cls._instanz = super().__call__(*args, **kwargs)
        return cls._instanz



class MethodenboxIntern(Ereignisbehandlung, Thread, metaclass=SingletonMeta):


    def __init__(self):
        """
        Erzeugt eine neue Methodenbox für das Arbeiten mit Bildern und Textfeldern
        """
        super().__init__()
        self.bilderliste = []
        self.textliste = []



    def fuegeBildHinzu(self, name, dateiangabe, xPos, yPos):
        """
        Hinzufügen eines Bildes
        --Parameter name eindeutiger Bezeichner des Bildes
        --Parameter dateiangabe füge das Bild im BlueJ-Projekt in den Ordner bilder ein und gib dann z. B. an "bilder/dateiname.png"
        --Parameter xPos x-Position der linken oberen Ecke des Bildes. 
        --Parameter yPos y-Position der linken oberen Ecke des Bildes
        """
    
        for b in self.bilderliste:
            if b[0] == name:
                print("Eindeutiger Bildname bereits vergeben. Bild wird nicht eingefügt")
                return
        self.bilderliste.append([name, Bild(xPos, yPos, 0, "rot", True, dateiangabe )])

    def entferneBild(self, name):
        """
        Methode zum Entfernen des Bildes
         --Parameter name der eindeutige Bezeichner des Bildes
        """    
        for b in self.bilderliste:
            if b[0] == name:
                b[1].SichtbarkeitSetzen(False)
                b[1].Entfernen()
                self.bilderliste.remove(b)
                return
 

    def ersetzeBild(self, name, dateiNeu):
        """
        Methode zum Ersetzen des Bildes durch eine andere Datei
        --Parameter name der eindeutige Bezeichner des Bildes
        --Parameter dateiNeu Pfad und Name der neuen Datei, z. B. "bilder/blume.jpg"
        """   
        for b in self.bilderliste:
            if b[0] == name:
                b[1].BildSetzen(dateiNeu)


    def setzeBildgroesse(self, name, breite, hoehe):
        """
        Methode zum Verändern der Bildgröße
        --Parameter name der eindeutige Bezeichner des Bildes
        --Parameter breite Breite des Bildes
        --Parameter hoehe Höhe des Bildes
        """   
        for b in self.bilderliste:
            if b[0] == name:
                b[1].GroesseSetzen(breite, hoehe)


    def setzeBildposition(self, name, xPos, yPos):
        """
        Methode zum Verändern der Position
        --Parameter name der eindeutige Bezeichner des Bildes
        --Parameter xPos neue x-Position der linken oberen Ecke des Bildes
        --Parameter yPos neue y-Position der linken oberen Ecke des Bildes
        """    
        for b in self.bilderliste:
            if b[0] == name:
                b[1].PositionSetzen(xPos, yPos)


    def fuegeTextHinzu(self, name, text, xPos, yPos):
        """
        Hinzufügen eines Textfeldes
        --Parameter name eindeutiger Bezeichner des Textfelds
        --Parameter text darzustellender Text
        --Parameter xPos x-Position der linken oberen Ecke des Textfeldes 
        --Parameter yPos y-Position der linken oberen Ecke des Textfeldes
        --Parameter breite Breite des Textfelds
        --Parameter hoehe Höhe des Textfelds
        """    
        for t in self.textliste:
            if t[0] == name:
                #print("Eindeutige Textname bereits vergeben. Text wird nicht eingefügt")
                return 
        textneu = Text(xPos, yPos, 0, 12, "schwarz", True)
        textneu.TextSetzen(text)
        self.textliste.append([name, textneu])
    


    def entferneText(self, name):
        """
        Methode zum Entfernen des Textfeldes
        --Parameter name der eindeutige Bezeichner des Textfelds
        """  
        for t in self.textliste:
            if t[0] == name:
                t[1].SichtbarkeitSetzen(False)
                t[1].Entfernen()
                self.textliste.remove(t)
                return
            

    def ersetzeText(self, name, textNeu):
        """
        Methode zum Ersetzen des Textes im Textfeld
        --Parameter name der eindeutige Bezeichner des Textfelds
        --Parameter textNeu der neue Text
        """    
        for t in self.textliste:
            if t[0] == name:
                t[1].TextSetzen(textNeu)
            

    def setzeTextposition(self, name, xPos, yPos):
        """
        Methode zum Ändern der Position des Textfeldes
        --Parameter name der eindeutige Bezeichner des Textfelds
        --Parameter xPos neue x-Position
        --Parameter yPos neue y-Position
        """   
        for t in self.textliste:
            if t[0] == name:     
                t[1].PositionSetzen(xPos, yPos)
            
        
    def TasteGedrueckt(self, taste):
        """
        Aktionsmethode, die bei jedem Tastendruck ausgelöst wird.
        Muss bei Bedarf von einer Unterklasse überschrieben werden.
        -- Parameter taste gedrückte Taste
        """
        from Methodenbox import reagiereAufTastendruck
        reagiereAufTastendruck(taste)